//
//  File.swift
//  
//
//  Created by Vasilis Akoinoglou on 20/10/21.
//

import SwiftUI

extension Path {
    static func interpolatePointsWithHermite(interpolationPoints: [CGPoint], alpha: CGFloat = 1.0 / 3.0) -> Path? {

        guard !interpolationPoints.isEmpty else { return nil }

        var path = Path()

        path.move(to: interpolationPoints[0])

        let n = interpolationPoints.count - 1

        for index in 0..<n {
            var currentPoint  = interpolationPoints[index]
            var nextIndex     = (index + 1) % interpolationPoints.count
            var prevIndex     = index == 0 ? interpolationPoints.count - 1 : index - 1
            var previousPoint = interpolationPoints[prevIndex]
            var nextPoint     = interpolationPoints[nextIndex]
            let endPoint      = nextPoint

            var mx: CGFloat
            var my: CGFloat

            if index > 0 {
                mx = (nextPoint.x - previousPoint.x) / 2.0
                my = (nextPoint.y - previousPoint.y) / 2.0
            } else {
                mx = (nextPoint.x - currentPoint.x) / 2.0
                my = (nextPoint.y - currentPoint.y) / 2.0
            }

            let controlPoint1 = CGPoint(x: currentPoint.x + mx * alpha, y: currentPoint.y + my * alpha)
            currentPoint      = interpolationPoints[nextIndex]
            nextIndex         = (nextIndex + 1) % interpolationPoints.count
            prevIndex         = index
            previousPoint     = interpolationPoints[prevIndex]
            nextPoint         = interpolationPoints[nextIndex]

            if index < n - 1 {
                mx = (nextPoint.x - previousPoint.x) / 2.0
                my = (nextPoint.y - previousPoint.y) / 2.0
            } else {
                mx = (currentPoint.x - previousPoint.x) / 2.0
                my = (currentPoint.y - previousPoint.y) / 2.0
            }

            let controlPoint2 = CGPoint(x: currentPoint.x - mx * alpha, y: currentPoint.y - my * alpha)

            path.addCurve(to: endPoint, control1: controlPoint1, control2: controlPoint2)
        }

        return path
    }
}
